"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValueByPath = exports.renderValues = exports.renderCondition = exports.generateAbility = exports.schemaRequestMetadata = exports.schemaBaseIdToken = exports._authTokenType = exports.AuthTokenType = void 0;
const joi_1 = __importDefault(require("joi"));
const lodash_1 = __importDefault(require("lodash"));
const ability_1 = require("@casl/ability");
const type_1 = require("../type");
const cm_libs_utils_1 = require("cm-libs-utils");
var AuthTokenType;
(function (AuthTokenType) {
    AuthTokenType["Bearer"] = "Bearer";
    AuthTokenType["Basic"] = "Basic";
})(AuthTokenType || (exports.AuthTokenType = AuthTokenType = {}));
exports._authTokenType = joi_1.default.string()
    .trim()
    .valid(...Object.values(AuthTokenType));
exports.schemaBaseIdToken = {
    userId: type_1._id.required(),
    userLogin: type_1._userLogin.required(),
    localeCode: type_1._localeCode.required(),
    orgId: type_1._id.allow(null),
    appDomain: type_1._appDomain.allow(null),
    productId: type_1._id.allow(null),
    extended: type_1._object.allow(null)
};
exports.schemaRequestMetadata = joi_1.default.object()
    .keys({
// Customized keys here
})
    .pattern(joi_1.default.string(), joi_1.default.any());
function generateAbility(rules, vars, fieldMatcher, conditionsMatcher) {
    if (!fieldMatcher) {
        fieldMatcher = (fields) => (field) => fields.includes(field);
    }
    if (!conditionsMatcher) {
        conditionsMatcher = (matchConditions) => matchConditions;
    }
    return new ability_1.PureAbility(rules || [], { fieldMatcher, conditionsMatcher });
}
exports.generateAbility = generateAbility;
function renderCondition(conditions, vars) {
    if (conditions) {
        for (const key of Object.keys(conditions)) {
            // Support max 10 variables
            for (let i = 0; 10 > i; i++) {
                const renderedValue = renderValue(conditions[key], 0, vars);
                if (undefined !== renderedValue) {
                    conditions[key] = renderedValue;
                    continue;
                }
                break;
            }
        }
    }
    return conditions;
}
exports.renderCondition = renderCondition;
function renderValues(object, vars) {
    if (!(0, cm_libs_utils_1.isUndefinedOrNull)(object)) {
        if (Array.isArray(object)) {
            const ret = [];
            for (const item of object) {
                ret.push(renderValues(item, vars));
            }
            return ret;
        }
        if ('object' === typeof object) {
            for (const key of Object.keys(object)) {
                object[key] = renderValues(object[key], vars);
            }
            return object;
        }
        const ret = renderValue(object, 0, vars);
        if (!(0, cm_libs_utils_1.isUndefinedOrNull)(ret) || ('string' === typeof object && object.startsWith('${') && object.endsWith('}'))) {
            return ret;
        }
    }
    return object;
}
exports.renderValues = renderValues;
function renderValue(value, offset, vars) {
    if ('string' === typeof value) {
        const open = value.indexOf('${', offset);
        if (0 <= open) {
            // This is nested placeholder
            const prefix = value.substring(0, open);
            const renderedValue = renderValue(value.substring(open), 2, vars);
            if (0 === prefix.length || ((0, cm_libs_utils_1.isUndefinedOrNull)(renderedValue) && 0 === prefix.length)) {
                return renderedValue;
            }
            value = prefix + (renderedValue || '');
        }
        const close = value.indexOf('}', 2);
        if (0 <= close) {
            const key = value.substring(2, close);
            if (key) {
                if (key.startsWith('`') && key.endsWith('`')) {
                    const script = key.substring(1, key.length - 1);
                    return eval(`(function (vars) {${script}})`)((key) => {
                        return 'function' === typeof vars ? vars(key) : getValueByPath(vars, key);
                    });
                }
                const renderedValue = 'function' === typeof vars ? vars(key) : getValueByPath(vars, key);
                const keyValue = `\${${key}}`;
                if ('string' === typeof renderedValue) {
                    return value.replace(keyValue, renderedValue);
                }
                else if ('number' === typeof renderedValue || 'boolean' === typeof renderedValue) {
                    // Return the whole value when exact match with key placeholder
                    return keyValue === value ? renderedValue : value.replace(keyValue, `${renderedValue}`);
                }
                else {
                    if ((0, cm_libs_utils_1.isUndefinedOrNull)(renderedValue)) {
                        return null;
                    }
                    return value.replace(keyValue, '');
                }
            }
        }
        return value;
    }
}
function getValueByPath(obj, path) {
    return lodash_1.default.get(obj, path);
}
exports.getValueByPath = getValueByPath;
//# sourceMappingURL=auth-type.js.map