"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getIp = exports.errorMiddleware = exports.finalMiddleware = exports.goodRes = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const config_base_1 = require("../config-base");
const correlation_id_1 = __importDefault(require("correlation-id"));
const lodash_1 = __importDefault(require("lodash"));
const slack_1 = require("../misc/slack");
function goodRes(res, data, statusCode, raw = false) {
    const resp = {
        data: undefined === data ? null : data
    };
    if (!raw) {
        if (statusCode) {
            res.status(statusCode).json(resp);
        }
        else {
            res.json(resp);
        }
    }
    else {
        if (statusCode) {
            res.status(statusCode).send(data);
        }
        else {
            res.send(data);
        }
    }
}
exports.goodRes = goodRes;
/* eslint-disable @typescript-eslint/no-unused-vars */
function finalMiddleware(req, res, next) {
    next((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.NOT_FOUND, 'Resource not found', { statusCode: 404 }));
}
exports.finalMiddleware = finalMiddleware;
function errorMiddleware(error, req, res, next) {
    const { code, message, reason } = (0, cm_libs_context_aware_error_1.describeError)(error);
    const SLACK_ERROR_CHANNEL = config_base_1.ConfigBase.get('SLACK_ERROR_CHANNEL', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString, ''));
    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
        const elapsed = req['start'] ? (0, cm_libs_utils_1.timeElapsed)(req['start']).toString() : '-';
        return `RES: ${req.method} ${req.url} ${res.statusCode}, elapsed: ${elapsed + ' ms'}, ERROR: ${reason}`;
    });
    if (!lodash_1.default.isEmpty(SLACK_ERROR_CHANNEL)) {
        const correlatorId = correlation_id_1.default.getId();
        const NODE_ENV = config_base_1.ConfigBase.get('NODE_ENV', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString, ''));
        const DISPLAY_SERVICE_NAME = config_base_1.ConfigBase.get('DISPLAY_SERVICE_NAME', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        let text = `Env: ${NODE_ENV}\n`;
        // text = text.concat(`Service: ${GlobalVariable.instance.getValue('name')}@${GlobalVariable.instance.getValue('version')}\n\n`)
        text = text.concat(`Correlator: ${correlatorId}\n`);
        text = text.concat(`API: RESTful\n`);
        if (DISPLAY_SERVICE_NAME) {
            text = text.concat(`Service: ${DISPLAY_SERVICE_NAME}\n`);
        }
        text = text.concat(`Date time: ${new Date().toISOString()}\n\n`);
        text = text.concat(`Reason: ${message}\n`);
        if (req.url && req.method)
            text = text.concat(`Path: ${req.method} ${req.url}\n\n`);
        if (reason)
            text = text.concat(`Stacktrace:\n\`\`\`${reason}\`\`\``);
        // ;(async () => {
        (0, slack_1.sendSlack)(text);
        //     console.log('data:', data)
        // })()
    }
    res.status(error.statusCode || 500).send({
        errors: [{ message, extensions: { code } }]
    });
}
exports.errorMiddleware = errorMiddleware;
function getIp(req) {
    // The X-Forwarded-For header might contains multiple values that separated by ","
    const xForwardedFor = req?.get('x-forwarded-for')?.split(',')?.[0] || req?.get('x-forwarded-for');
    return (xForwardedFor || req?.get('x-real-ip') || '0.0.0.0');
}
exports.getIp = getIp;
//# sourceMappingURL=http-handler.js.map